@@ -1,27 +1,52 @@
 package Scientist;
-$Scientist::VERSION = '0.007';
+
 use strict;
 use warnings;
+our $VERSION = '0.008'; # VERSION
 
 use Moo;
 use Test::Deep::NoTest qw/deep_diag cmp_details/;
 use Time::HiRes 'time';
+use Types::Standard qw/Bool Str CodeRef HashRef/;
 
 # ABSTRACT: Perl module inspired by https://github.com/github/scientist
 # https://github.com/lancew/Scientist
 
-# TODO: Should limit to a hashref
-has 'context' => ( is => 'rw' );
+has 'context' => (
+    is       => 'rw',
+    isa      => HashRef,
+);
 
-has 'enabled' => ( is => 'rw', default => 1 );
+has 'enabled' => (
+    is       => 'rw',
+    isa      => Bool,
+    default  => 1,
+);
 
-has 'experiment' => ( is => 'rw' );
+has 'experiment' => (
+    is       => 'ro',
+    isa      => Str,
+    builder  => 'name',
+);
 
-has 'use' => ( is => 'rw' );
+has 'use' => (
+    is       => 'rw',
+    isa      => CodeRef,
+);
 
-has 'result' => ( is => 'rw', );
+has 'result' => (
+    is       => 'rw',
+    isa      => HashRef,
+);
 
-has 'try' => ( is => 'rw' );
+has 'try' => (
+    is       => 'rw',
+    isa      => CodeRef,
+);
+
+sub name {
+    return 'experiment';
+}
 
 sub publish {
     my $self = shift;
@@ -33,7 +58,7 @@ sub publish {
 sub run {
     my $self = shift;
 
-    # If experiement not enabled just return the control code results.
+    # If experiment not enabled just return the control code results.
     return $self->use->() unless $self->enabled;
 
     my %result = (
@@ -45,15 +70,17 @@ sub run {
 
     my ( @control, @candidate );
     my $run_control = sub {
-        my $start = Time::HiRes::time;
+        my $start = time;
         @control = $wantarray ? $self->use->() : scalar $self->use->();
-        $result{control}{duration} = Time::HiRes::time - $start;
+        $result{control}{duration} = time - $start;
     };
 
     my $run_candidate = sub {
-        my $start = Time::HiRes::time;
-        @candidate = $wantarray ? $self->try->() : scalar $self->try->();
-        $result{candidate}{duration} = Time::HiRes::time - $start;
+        my $start = time;
+        @candidate = $wantarray
+            ? eval { $self->try->() }
+            : eval { scalar $self->try->() };
+        $result{candidate}{duration} = time - $start;
     };
 
     if ( rand > 0.5 ) {
